Package: spuRs
Type: Package
Title: Functions and Datasets for "Introduction to Scientific
        Programming and Simulation Using R"
Version: 2.0.3
Date: 2025-04-27
Authors@R: c(person(given = "Owen",
                      family = "Jones",
                      role = "aut"),
               person(given = "Robert",
                      family = "Maillardet",
                      role = "aut"),
               person(given = "Andrew",
                      family = "Robinson",
                      role = c("aut", "cre"),
                      email = "apro@unimelb.edu.au"),
               person(given = "Olga",
                      family = "Borovkova",
                      role = "aut"),
               person(given = "Steven",
                      family = "Carnie",
                      role = "aut"))
Maintainer: Andrew Robinson <apro@unimelb.edu.au>
Depends: R (>= 2.10), MASS, lattice
Description: Provides functions and datasets from Jones, O.D., R. Maillardet, and A.P. Robinson.  2014.  An Introduction to Scientific Programming and Simulation, Using R. 2nd Ed. Chapman And Hall/CRC.
License: GPL-3
LazyLoad: yes
NeedsCompilation: no
Packaged: 2025-04-27 12:35:12 UTC; apro
Author: Owen Jones [aut],
  Robert Maillardet [aut],
  Andrew Robinson [aut, cre],
  Olga Borovkova [aut],
  Steven Carnie [aut]
Repository: CRAN
Date/Publication: 2025-04-27 13:10:01 UTC
Built: R 4.6.0; ; 2026-04-25 22:33:22 UTC; unix
